class D
   Real x;
   parameter Real y=4;
   
equation
   if y >= 20 then
      x = 5;
   end if;
   if y <= 20 then
      x = 10;
   end if;
end D;

model C
   Real x;
   Real y;
   constant Real u=7;
   parameter Real high=12;
   parameter Real low=3;
   
equation
   y = if u > high then
      high
   else
      if u < low then
         low
      else
         u;
   
algorithm
   when time <= 2 then
      x := 4;
   end when;
   when x <= y then
      x := 6;
   end when;
end C;

class DivideCall
   Real divide;
   
equation
   divide = 12/10;
end DivideCall;

function Divide
   input Integer numerator;
   output Real result;
   input Real denominator;
   
algorithm
   result := numerator/denominator;
end Divide;

class DoubleCall
   Real double;
   
equation
   double = Double(12);
end DoubleCall;

function Double
   input Real x;
   output Real y;
   
algorithm
   y := 2*x;
end Double;

class InheritsContainer
   extends Container(redeclare replaceable FlyingPlane plane1(redeclare parameter MyReal mass,velocity=200));
end InheritsContainer;

class Container
   replaceable Plane plane1(mass=100);
end Container;

type MyReal = Real;

class FlyingPlane
   extends Plane(velocity=100);
   Real altitude=11000;
end FlyingPlane;

class Plane
   replaceable Real mass;
   Real velocity=0;
end Plane;

class Vehicle
   Real mass;
end Vehicle;

class BubbleSortSimulation
   BubbleSort b(elements[:]={31231,231,3493,9349,8985,586,69,34,9,122});
end BubbleSortSimulation;

class BubbleSort
   parameter Real[:] elements;
   Real sorted[size(elements,1)];
   function bubblesortfunc
      input Real[:] unorderedElements;
      output Real[size(unorderedElements,1)] orderedElements;
      
   protected
      Real temporaryValue;
      Boolean isOver=false;
      
   algorithm
      orderedElements := unorderedElements;
      while not isOver loop
         isOver := true;
         for i in 1:size(orderedElements,1)-1 loop
            if orderedElements[i] > orderedElements[i+1] then
               temporaryValue := orderedElements[i];
               orderedElements[i] := orderedElements[i+1];
               orderedElements[i+1] := temporaryValue;
               isOver := false;
            end if;
         end for;
      end while;
   end bubblesortfunc;
   
algorithm
   sorted := bubblesortfunc(elements);
end BubbleSort;

model CollidingSpheres "Collision modeling class"
   parameter Real distance=1 "the distance between spheres";
   parameter Real e=0.2 "the restitution coefficient";
   parameter Real mass=1 "mass of each sphere";
   parameter Real diameter=1 "diameter of each sphere";
   parameter Real velocity=1 "initial collision of s1 with s2";
   parameter Real m1=mass;
   parameter Real m2=mass;
   parameter Real m3=mass;
   parameter Real m4=mass;
   Real x1(start=0);
   Real x2;
   Real x3;
   Real x4;
   Real y1(start=distance);
   Real y2(start=distance);
   Real y3(start=distance);
   Real x1der(start=velocity);
   Real y1der(start=-velocity);
   Real y2der(start=0);
   Real y3der(start=0);
   Real x1derder(start=0);
   Real y1derder(start=0);
   Real y2derder(start=0);
   Real y3derder(start=0);
   
equation
   y1 = (x2-x1)-distance;
   y2 = (x3-x2)-distance;
   y3 = (x4-x3)-distance;
   x1der = der(x1);
   y1der = der(y1);
   y2der = der(y2);
   y3der = der(y3);
   x1derder = der(x1der);
   y1derder = der(y1der);
   y2derder = der(y2der);
   y3derder = der(y3der);
   x1derder = 0;
   y1derder = 0;
   y2derder = 0;
   y3derder = 0;
   when x2-x1 <= diameter then
      reinit(x1der,pre(x1der)+((1+e)*m2)/(m1+m2)*pre(y1der));
      reinit(y2der,pre(y2der)+((1+e)*m1)/(m1+m2)*pre(y1der));
      reinit(y1der,-e*pre(y1der));
   end when;
   when x3-x2 <= diameter then
      reinit(y1der,pre(y1der)+((1+e)*m3)/(m2+m3)*pre(y2der));
      reinit(y3der,pre(y3der)+((1+e)*m2)/(m2+m3)*pre(y2der));
      reinit(y2der,-e*pre(y2der));
   end when;
   when x4-x3 <= diameter then
      reinit(y2der,pre(y2der)+((1+e)*m4)/(m3+m4)*pre(y3der));
      reinit(y3der,-e*pre(y3der));
   end when;
end CollidingSpheres;

class Exercise2withGear2 = Exercise2withSpring(redeclare Modelica.Mechanics.Rotational.IdealGear spring(ratio=100));

class Exercise2withGear "Modelica Exam Exercise 2 with ideal gear"
   extends Exercise2;
   replaceable Modelica.Mechanics.Rotational.IdealGear spring(ratio=3);
   
equation
   connect(inertia.flange_b, spring.flange_a);
   connect(spring.flange_b, joint1.axis);
end Exercise2withGear;

class Exercise2withDumping = Exercise2withSpring(redeclare Modelica.Mechanics.Rotational.SpringDamper spring(c=300,phi_rel0=1,d=100));

class Exercise2withSpring "Modelica Exam Exercise 2 with spring"
   extends Exercise2;
   replaceable Modelica.Mechanics.Rotational.Spring spring(c=300);
   
equation
   connect(inertia.flange_b, spring.flange_a);
   connect(spring.flange_b, joint1.axis);
end Exercise2withSpring;

partial model Exercise2 "Modelica Exam Exercise 2"
   parameter Real length1=2 "Length of the first arm";
   parameter Real length2=3 "Length of the second arm";
   parameter Real mass1=1 "Mass of the first arm";
   parameter Real mass2=1 "Mass of the second arm";
   Modelica.Electrical.Analog.Basic.Ground ground;
   Modelica.Electrical.Analog.Basic.Resistor resistor;
   Modelica.Electrical.Analog.Basic.EMF emf;
   Modelica.Mechanics.Rotational.Inertia inertia;
   ModelicaAdditions.MultiBody.Parts.InertialSystem inertialsystem;
   ModelicaAdditions.MultiBody.Parts.Body arm2(rCM={length2/2,0,0},m=mass2);
   ModelicaAdditions.MultiBody.Parts.Body2 arm1(r={length1,0,0},rCM={length1/2,0,0},m=mass1);
   ModelicaAdditions.MultiBody.Joints.Revolute joint1(n={0,0,1});
   ModelicaAdditions.MultiBody.Joints.Revolute joint2(n={0,0,1});
   Modelica.Electrical.Analog.Sources.SineVoltage sinevoltage;
   
equation
   connect(emf.p, resistor.n);
   connect(ground.p, emf.n);
   connect(emf.flange_b, inertia.flange_a);
   connect(inertialsystem.frame_b, joint1.frame_a);
   connect(joint1.frame_b, arm1.frame_a);
   connect(arm1.frame_b, joint2.frame_a);
   connect(arm2.frame_a, joint2.frame_b);
   connect(sinevoltage.n, resistor.p);
   connect(sinevoltage.p, ground.p);
end Exercise2;

class SimulateLorentz
   Lorentz lorentz1;
   Lorentz lorentz2(sigma=1,b=10,R=1);
   Lorentz lorentz1SC(x(start=1.1),y(start=1.1),z(start=1.1));
   Lorentz lorentz2SC(sigma=1,b=10,R=1,x(start=1.1),y(start=1.1),z(start=1.1));
end SimulateLorentz;

class Lorentz
   parameter Real sigma=10;
   parameter Real b=8/3;
   parameter Real R=28;
   Real x(start=1);
   Real y(start=1);
   Real z(start=1);
   
equation
   der(x) = -sigma*x+sigma*y;
   der(y) = (R*x-y)-x*z;
   der(z) = -b*x+x*y;
end Lorentz;